@@ -38,6 +38,7 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "handbrake/qsv_memory.h"
 #include "handbrake/h264_common.h"
 #include "handbrake/h265_common.h"
+#include "handbrake/extradata.h"
 
 #include "libavutil/hwcontext_qsv.h"
 #include "libavutil/hwcontext.h"
@@ -151,11 +152,6 @@ static int64_t get_frame_duration(hb_work_private_t *pv, hb_buffer_t *buf)
     return pv->frame_duration[i];
 }
 
-static inline int64_t rescale(hb_rational_t q, int b)
-{
-    return av_rescale(q.num, b, q.den);
-}
-
 static const char* hyper_encode_name(const int hyper_encode_mode)
 {
     switch (hyper_encode_mode)
@@ -648,7 +644,9 @@ static int qsv_hevc_make_header(hb_work_object_t *w, mfxSession session, const m
     len = bitstream.DataLength;
     buf = bitstream.Data + bitstream.DataOffset;
     end = bitstream.Data + bitstream.DataOffset + bitstream.DataLength;
-    w->config->h265.headers_length = 0;
+
+    size_t extradata_size = 0;
+    uint8_t extradata[HB_CONFIG_MAX_SIZE];
 
     while ((buf = hb_annexb_find_next_nalu(buf, &len)) != NULL)
     {
@@ -665,19 +663,20 @@ static int qsv_hevc_make_header(hb_work_object_t *w, mfxSession session, const m
                 continue;
         }
 
-        size_t size = hb_nal_unit_write_annexb(NULL, buf, len) + w->config->h265.headers_length;
-        if (sizeof(w->config->h265.headers) < size)
+        size_t size = hb_nal_unit_write_annexb(NULL, buf, len) + extradata_size;
+        if (sizeof(extradata) < size)
         {
             /* Will never happen in practice */
             hb_log("qsv_hevc_make_header: header too large (size: %lu, max: %lu)",
-                   size, sizeof(w->config->h265.headers));
+                   size, sizeof(extradata));
         }
 
-        w->config->h265.headers_length += hb_nal_unit_write_annexb(w->config->h265.headers +
-                                                                   w->config->h265.headers_length, buf, len);
+        extradata_size += hb_nal_unit_write_annexb(extradata + extradata_size, buf, len);
         len = end - buf;
     }
 
+    hb_set_extradata(w->extradata, extradata, extradata_size);
+
 end:
     if (bitstream.Data)
         av_free(bitstream.Data);
@@ -1215,16 +1214,16 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
             if (job->mastering.has_primaries && job->mastering.has_luminance)
             {
                 pv->param.masteringDisplayColourVolume.InsertPayloadToggle = MFX_PAYLOAD_IDR;
-                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[0] = FFMIN(rescale(job->mastering.display_primaries[1][0], masteringChromaDen), masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[0] = FFMIN(rescale(job->mastering.display_primaries[1][1], masteringChromaDen), masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[1] = FFMIN(rescale(job->mastering.display_primaries[2][0], masteringChromaDen), masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[1] = FFMIN(rescale(job->mastering.display_primaries[2][1], masteringChromaDen), masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[2] = FFMIN(rescale(job->mastering.display_primaries[0][0], masteringChromaDen), masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[2] = FFMIN(rescale(job->mastering.display_primaries[0][1], masteringChromaDen), masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.WhitePointX = FFMIN(rescale(job->mastering.white_point[0], masteringChromaDen), masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.WhitePointY = FFMIN(rescale(job->mastering.white_point[1], masteringChromaDen), masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.MaxDisplayMasteringLuminance = rescale(job->mastering.max_luminance, masteringLumaDen);
-                pv->param.masteringDisplayColourVolume.MinDisplayMasteringLuminance = FFMIN(rescale(job->mastering.min_luminance, masteringLumaDen),
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[0] = FFMIN(hb_rescale_rational(job->mastering.display_primaries[1][0], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[0] = FFMIN(hb_rescale_rational(job->mastering.display_primaries[1][1], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[1] = FFMIN(hb_rescale_rational(job->mastering.display_primaries[2][0], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[1] = FFMIN(hb_rescale_rational(job->mastering.display_primaries[2][1], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[2] = FFMIN(hb_rescale_rational(job->mastering.display_primaries[0][0], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[2] = FFMIN(hb_rescale_rational(job->mastering.display_primaries[0][1], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.WhitePointX = FFMIN(hb_rescale_rational(job->mastering.white_point[0], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.WhitePointY = FFMIN(hb_rescale_rational(job->mastering.white_point[1], masteringChromaDen), masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.MaxDisplayMasteringLuminance = hb_rescale_rational(job->mastering.max_luminance, masteringLumaDen);
+                pv->param.masteringDisplayColourVolume.MinDisplayMasteringLuminance = FFMIN(hb_rescale_rational(job->mastering.min_luminance, masteringLumaDen),
                     pv->param.masteringDisplayColourVolume.MaxDisplayMasteringLuminance);
             }
 
@@ -1246,16 +1245,16 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
             if (job->mastering.has_primaries && job->mastering.has_luminance)
             {
                 pv->param.masteringDisplayColourVolume.InsertPayloadToggle = MFX_PAYLOAD_IDR;
-                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[0] = rescale(job->mastering.display_primaries[0][0], masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[0] = rescale(job->mastering.display_primaries[0][1], masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[1] = rescale(job->mastering.display_primaries[1][0], masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[1] = rescale(job->mastering.display_primaries[1][1], masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[2] = rescale(job->mastering.display_primaries[2][0], masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[2] = rescale(job->mastering.display_primaries[2][1], masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.WhitePointX = rescale(job->mastering.white_point[0], masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.WhitePointY = rescale(job->mastering.white_point[1], masteringChromaDen);
-                pv->param.masteringDisplayColourVolume.MaxDisplayMasteringLuminance = rescale(job->mastering.max_luminance, max_luma_den);
-                pv->param.masteringDisplayColourVolume.MinDisplayMasteringLuminance = rescale(job->mastering.min_luminance, min_luma_den);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[0] = hb_rescale_rational(job->mastering.display_primaries[0][0], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[0] = hb_rescale_rational(job->mastering.display_primaries[0][1], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[1] = hb_rescale_rational(job->mastering.display_primaries[1][0], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[1] = hb_rescale_rational(job->mastering.display_primaries[1][1], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesX[2] = hb_rescale_rational(job->mastering.display_primaries[2][0], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.DisplayPrimariesY[2] = hb_rescale_rational(job->mastering.display_primaries[2][1], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.WhitePointX = hb_rescale_rational(job->mastering.white_point[0], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.WhitePointY = hb_rescale_rational(job->mastering.white_point[1], masteringChromaDen);
+                pv->param.masteringDisplayColourVolume.MaxDisplayMasteringLuminance = hb_rescale_rational(job->mastering.max_luminance, max_luma_den);
+                pv->param.masteringDisplayColourVolume.MinDisplayMasteringLuminance = hb_rescale_rational(job->mastering.min_luminance, min_luma_den);
             }
 
             /*  Content light level */
@@ -1714,16 +1713,20 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     memset(&videoParam, 0, sizeof(mfxVideoParam));
     videoParam.ExtParam = extParamArray;
     videoParam.NumExtParam = 0;
+
+    uint8_t sps[HB_CONFIG_MAX_SIZE];
+    uint8_t pps[HB_CONFIG_MAX_SIZE];
+
     // introduced in API 1.3
     memset(sps_pps, 0, sizeof(mfxExtCodingOptionSPSPPS));
     sps_pps->Header.BufferId = MFX_EXTBUFF_CODING_OPTION_SPSPPS;
     sps_pps->Header.BufferSz = sizeof(mfxExtCodingOptionSPSPPS);
     sps_pps->SPSId           = 0;
-    sps_pps->SPSBuffer       = w->config->h264.sps;
-    sps_pps->SPSBufSize      = sizeof(w->config->h264.sps);
+    sps_pps->SPSBuffer       = sps;
+    sps_pps->SPSBufSize      = sizeof(sps);
     sps_pps->PPSId           = 0;
-    sps_pps->PPSBuffer       = w->config->h264.pps;
-    sps_pps->PPSBufSize      = sizeof(w->config->h264.pps);
+    sps_pps->PPSBuffer       = pps;
+    sps_pps->PPSBufSize      = sizeof(pps);
     if (pv->param.videoParam->mfx.CodecId == MFX_CODEC_AVC)
     {
         videoParam.ExtParam[videoParam.NumExtParam++] = (mfxExtBuffer*)sps_pps;
@@ -1793,12 +1796,9 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     if (videoParam.mfx.CodecId == MFX_CODEC_AVC)
     {
         // remove 4-byte Annex B NAL unit prefix (0x00 0x00 0x00 0x01)
-        w->config->h264.sps_length = sps_pps->SPSBufSize - 4;
-        memmove(w->config->h264.sps, w->config->h264.sps + 4,
-                w->config->h264.sps_length);
-        w->config->h264.pps_length = sps_pps->PPSBufSize - 4;
-        memmove(w->config->h264.pps, w->config->h264.pps + 4,
-                w->config->h264.pps_length);
+        hb_set_h264_extradata(w->extradata,
+                              sps + 4, sps_pps->SPSBufSize - 4,
+                              pps + 4, sps_pps->PPSBufSize - 4);
     }
     else if (videoParam.mfx.CodecId == MFX_CODEC_HEVC)
     {
@@ -1832,7 +1832,7 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         {
             case MFX_CODEC_AVC:
             case MFX_CODEC_HEVC:
-                pv->init_delay = &w->config->init_delay;
+                pv->init_delay = w->init_delay;
                 break;
             default:
                 break;
