@@ -1,6 +1,6 @@
 /* encavcodec.c
 
-   Copyright (c) 2003-2023 HandBrake Team
+   Copyright (c) 2003-2024 HandBrake Team
    Copyright 2022 NVIDIA Corporation
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
@@ -17,6 +17,8 @@
 #include "handbrake/av1_common.h"
 #include "handbrake/nal_units.h"
 #include "handbrake/nvenc_common.h"
+#include "handbrake/vce_common.h"
+#include "handbrake/extradata.h"
 
 /*
  * The frame info struct remembers information about each frame across calls
@@ -56,11 +58,18 @@ int  encavcodecInit( hb_work_object_t *, hb_job_t * );
 int  encavcodecWork( hb_work_object_t *, hb_buffer_t **, hb_buffer_t ** );
 void encavcodecClose( hb_work_object_t * );
 
-static int apply_encoder_preset(int vcodec, AVDictionary ** av_opts,
-                                const char * preset);
+static int apply_encoder_preset(int vcodec, AVCodecContext *context,
+                                AVDictionary **av_opts,
+                                const char *preset);
+static int apply_encoder_tune(int vcodec, AVDictionary ** av_opts,
+                                const char * tune);
+
 static int apply_encoder_options(hb_job_t *job, AVCodecContext *context,
                                  AVDictionary **av_opts);
 
+static int apply_encoder_level(AVCodecContext *context, AVDictionary **av_opts,
+                               int vcodec, const char *encoder_level);
+
 hb_work_object_t hb_encavcodec =
 {
     WORK_ENCAVCODEC,
@@ -75,11 +84,21 @@ static const char * const vpx_preset_names[] =
     "veryfast", "faster", "fast", "medium", "slow", "slower", "veryslow", NULL
 };
 
+static const char * const vp9_tune_names[] = 
+{
+    "screen", "film", NULL 
+};
+
 static const char * const h26x_nvenc_preset_names[] =
 {
     "fastest", "faster", "fast", "medium", "slow", "slower", "slowest", NULL
 };
 
+static const char * const ffv1_preset_names[] =
+{
+    "default", "preservation", NULL
+};
+
 static const char * const h264_nvenc_profile_names[] =
 {
     "auto", "baseline", "main", "high", NULL  // "high444p" not supported.
@@ -110,6 +129,21 @@ static const char * const h265_mf_profile_name[] =
     "auto", "main",  NULL
 };
 
+static const char * const ffv1_profile_names[] =
+{
+    "auto", NULL
+};
+
+static const char * const hb_ffv1_level_names[] =
+{
+    "auto", "1", "3", NULL
+};
+
+static const int hb_ffv1_level_values[] =
+{
+    -1,  1,  3,  0
+};
+
 static const enum AVPixelFormat standard_pix_fmts[] =
 {
     AV_PIX_FMT_YUV420P, AV_PIX_FMT_NONE
@@ -140,6 +174,13 @@ static const enum AVPixelFormat vce_pix_formats_10bit[] =
      AV_PIX_FMT_P010, AV_PIX_FMT_NONE
 };
 
+static const enum AVPixelFormat ffv1_pix_formats[] =
+{
+    AV_PIX_FMT_YUV444P16, AV_PIX_FMT_YUV444P12, AV_PIX_FMT_YUV444P10, AV_PIX_FMT_YUV444P,
+    AV_PIX_FMT_YUV422P16, AV_PIX_FMT_YUV422P12, AV_PIX_FMT_YUV422P10, AV_PIX_FMT_YUV422P,
+    AV_PIX_FMT_YUV420P16, AV_PIX_FMT_YUV420P12, AV_PIX_FMT_YUV420P10, AV_PIX_FMT_YUV420P,
+    AV_PIX_FMT_NONE
+};
 
 int encavcodecInit( hb_work_object_t * w, hb_job_t * job )
 {
@@ -149,6 +190,7 @@ int encavcodecInit( hb_work_object_t * w, hb_job_t * job )
     const AVCodec * codec = NULL;
     AVCodecContext * context;
     AVRational fps;
+    AVDictionary *av_opts = NULL;
 
     hb_work_private_t * pv = calloc( 1, sizeof( hb_work_private_t ) );
     w->private_data   = pv;
@@ -240,6 +282,15 @@ int encavcodecInit( hb_work_object_t * w, hb_job_t * job )
                     break;
             }
         }break;
+        case AV_CODEC_ID_FFV1:
+        {
+            switch (job->vcodec) {
+                case HB_VCODEC_FFMPEG_FFV1:
+                    hb_log("encavcodecInit: FFV1 (libavcodec)");
+                    codec_name = "ffv1";
+                    break;
+            }
+        }break;
     }
 
     if (codec_name == NULL)
@@ -321,32 +372,28 @@ int encavcodecInit( hb_work_object_t * w, hb_job_t * job )
     context->framerate     = fps;
     context->gop_size  = ((double)job->orig_vrate.num / job->orig_vrate.den +
                                   0.5) * 10;
-    if ((job->vcodec == HB_VCODEC_FFMPEG_VCE_H264)
-        || (job->vcodec == HB_VCODEC_FFMPEG_VCE_H265)
-        || (job->vcodec == HB_VCODEC_FFMPEG_VCE_H265_10BIT)
-        || (job->vcodec == HB_VCODEC_FFMPEG_VCE_AV1))
+
+    if (apply_encoder_level(context, &av_opts, job->vcodec, job->encoder_level))
     {
-        // Set encoder preset
-        context->profile = FF_PROFILE_UNKNOWN;
-        if (job->encoder_preset != NULL && *job->encoder_preset)
-        {
-            if ((!strcasecmp(job->encoder_preset, "balanced"))
-                || (!strcasecmp(job->encoder_preset, "speed"))
-                || (!strcasecmp(job->encoder_preset, "quality")))
-            {
-                av_opt_set(context, "quality", job->encoder_preset, AV_OPT_SEARCH_CHILDREN);
-            }
-        }
+        av_free(context);
+        ret = 1;
+        goto done;
     }
 
-    AVDictionary * av_opts = NULL;
-    if (apply_encoder_preset(job->vcodec, &av_opts, job->encoder_preset))
+    if (apply_encoder_preset(job->vcodec, context, &av_opts, job->encoder_preset))
     {
         av_free( context );
         ret = 1;
         goto done;
     }
 
+    if (apply_encoder_tune(job->vcodec, &av_opts, job->encoder_tune))
+    {
+        av_free(context);
+        ret = 1;
+        goto done;
+    }
+
     if (apply_encoder_options(job, context, &av_opts))
     {
         av_free( context );
@@ -598,7 +645,6 @@ int encavcodecInit( hb_work_object_t * w, hb_job_t * job )
 
     if (job->vcodec == HB_VCODEC_FFMPEG_VCE_H264)
     {
-        // Set profile and level
         context->profile = FF_PROFILE_UNKNOWN;
         if (job->encoder_profile != NULL && *job->encoder_profile)
         {
@@ -609,22 +655,9 @@ int encavcodecInit( hb_work_object_t * w, hb_job_t * job )
             else if (!strcasecmp(job->encoder_profile, "high"))
                 context->profile = FF_PROFILE_H264_HIGH;
         }
-        context->level = FF_LEVEL_UNKNOWN;
-        if (job->encoder_level != NULL && *job->encoder_level)
-        {
-            int i = 1;
-            while (hb_h264_level_names[i] != NULL)
-            {
-                if (!strcasecmp(job->encoder_level, hb_h264_level_names[i]))
-                    context->level = hb_h264_level_values[i];
-                ++i;
-            }
-        }
     }
-
-    if (job->vcodec == HB_VCODEC_FFMPEG_VCE_H265 || job->vcodec == HB_VCODEC_FFMPEG_VCE_H265_10BIT)
+    else if (job->vcodec == HB_VCODEC_FFMPEG_VCE_H265 || job->vcodec == HB_VCODEC_FFMPEG_VCE_H265_10BIT)
     {
-        // Set profile and level
         context->profile = FF_PROFILE_UNKNOWN;
         if (job->encoder_profile != NULL && *job->encoder_profile)
         {
@@ -637,52 +670,27 @@ int encavcodecInit( hb_work_object_t * w, hb_job_t * job )
             }
         }
 
-        context->level = FF_LEVEL_UNKNOWN;
-        if (job->encoder_level != NULL && *job->encoder_level)
-        {
-            int i = 1;
-            while (hb_h265_level_names[i] != NULL)
-            {
-                if (!strcasecmp(job->encoder_level, hb_h265_level_names[i]))
-                    context->level = hb_h265_level_values[i];
-                ++i;
-            }
-        }
-        // FIXME
-        //context->tier = FF_TIER_UNKNOWN;
+        // Make VCE h.265 encoder emit an IDR for every GOP
+        av_dict_set(&av_opts, "gops_per_idr", "1", 0);
     }
-    if (job->vcodec == HB_VCODEC_FFMPEG_VCE_AV1)
+    else if (job->vcodec == HB_VCODEC_FFMPEG_VCE_AV1)
     {
-        // Set profile and level
         context->profile = FF_PROFILE_UNKNOWN;
         if (job->encoder_profile != NULL && *job->encoder_profile)
         {
             if (!strcasecmp(job->encoder_profile, "main"))
                  context->profile = FF_PROFILE_AV1_MAIN;
         }
-        context->level = FF_LEVEL_UNKNOWN;
-        if (job->encoder_level != NULL && *job->encoder_level)
-        {
-            int i = 1;
-            while (hb_av1_level_names[i] != NULL)
-            {
-                if (!strcasecmp(job->encoder_level, hb_av1_level_names[i]))
-                    context->level = hb_av1_level_values[i];
-                ++i;
-            }
-        }
     }
-
-    if (job->vcodec == HB_VCODEC_FFMPEG_NVENC_H264 ||
-        job->vcodec == HB_VCODEC_FFMPEG_NVENC_H265 ||
-        job->vcodec == HB_VCODEC_FFMPEG_NVENC_H265_10BIT ||
-        job->vcodec == HB_VCODEC_FFMPEG_NVENC_AV1 ||
-        job->vcodec == HB_VCODEC_FFMPEG_NVENC_AV1_10BIT)
+    else if (job->vcodec == HB_VCODEC_FFMPEG_NVENC_H264 ||
+             job->vcodec == HB_VCODEC_FFMPEG_NVENC_H265 ||
+             job->vcodec == HB_VCODEC_FFMPEG_NVENC_H265_10BIT ||
+             job->vcodec == HB_VCODEC_FFMPEG_NVENC_AV1 ||
+             job->vcodec == HB_VCODEC_FFMPEG_NVENC_AV1_10BIT)
     {
         // Force IDR frames when we force a new keyframe for chapters
         av_dict_set( &av_opts, "forced-idr", "1", 0 );
 
-        // Set profile and level
         if (job->encoder_profile != NULL && *job->encoder_profile)
         {
             if (!strcasecmp(job->encoder_profile, "baseline"))
@@ -693,17 +701,6 @@ int encavcodecInit( hb_work_object_t * w, hb_job_t * job )
                 av_dict_set(&av_opts, "profile", "high", 0);
         }
 
-        if (job->encoder_level != NULL && *job->encoder_level)
-        {
-            int i = 1;
-            while (hb_h264_level_names[i] != NULL)
-            {
-                if (!strcasecmp(job->encoder_level, hb_h264_level_names[i]))
-                    av_dict_set(&av_opts, "level", job->encoder_level, 0);
-                ++i;
-            }
-        }
-
         // Disable unhandled SEI
         // These might be present in the source video
         // and passed through in side_data, but we
@@ -711,42 +708,47 @@ int encavcodecInit( hb_work_object_t * w, hb_job_t * job )
         av_dict_set(&av_opts, "a53cc", "0", 0);
         av_dict_set(&av_opts, "s12m_tc", "0", 0);
     }
-
-    // Make VCE h.265 encoder emit an IDR for every GOP
-    if (job->vcodec == HB_VCODEC_FFMPEG_VCE_H265 || job->vcodec == HB_VCODEC_FFMPEG_VCE_H265_10BIT)
+    else if (job->vcodec == HB_VCODEC_FFMPEG_FFV1)
     {
-        av_dict_set(&av_opts, "gops_per_idr", "1", 0);
-    }
+        int slices[] = {4, 6, 9, 12, 16, 24, 30};
+        context->slices = hb_get_cpu_count();
 
-    if (job->vcodec == HB_VCODEC_FFMPEG_MF_H264)
-    {
-        context->profile = FF_PROFILE_UNKNOWN;
-        if (job->encoder_profile != NULL && *job->encoder_profile)
+        int slice_index = 0;
+        for (int i = 0; i < sizeof(slices) / sizeof(int); i++)
         {
-            if (!strcasecmp(job->encoder_profile, "baseline"))
-                context->profile = FF_PROFILE_H264_BASELINE;
-            else if (!strcasecmp(job->encoder_profile, "main"))
-                 context->profile = FF_PROFILE_H264_MAIN;
-            else if (!strcasecmp(job->encoder_profile, "high"))
-                context->profile = FF_PROFILE_H264_HIGH;
+            if (context->slices >= slices[i])
+            {
+                slice_index = i;
+            }
         }
-
+        context->slices = slices[slice_index];
     }
-
-    if (job->vcodec == HB_VCODEC_FFMPEG_MF_H264 ||
-        job->vcodec == HB_VCODEC_FFMPEG_MF_H265)
+    else if (job->vcodec == HB_VCODEC_FFMPEG_MF_H264 ||
+             job->vcodec == HB_VCODEC_FFMPEG_MF_H265)
     {
+        if (job->vcodec == HB_VCODEC_FFMPEG_MF_H264)
+        {
+            context->profile = FF_PROFILE_UNKNOWN;
+            if (job->encoder_profile != NULL && *job->encoder_profile)
+            {
+                if (!strcasecmp(job->encoder_profile, "baseline"))
+                    context->profile = FF_PROFILE_H264_BASELINE;
+                else if (!strcasecmp(job->encoder_profile, "main"))
+                    context->profile = FF_PROFILE_H264_MAIN;
+                else if (!strcasecmp(job->encoder_profile, "high"))
+                    context->profile = FF_PROFILE_H264_HIGH;
+            }
+        }
+        else if (job->vcodec == HB_VCODEC_FFMPEG_MF_H265)
+        {
+            // Qualcomm's HEVC encoder does support b-frames. Some chipsets
+            // support setting this to either 1 or 2, while others only support
+            // setting it to 1.
+            context->max_b_frames = 1;
+        }
         av_dict_set(&av_opts, "hw_encoding", "1", 0);
     }
 
-    if (job->vcodec == HB_VCODEC_FFMPEG_MF_H265)
-    {
-        // Qualcomm's HEVC encoder does support b-frames. Some chipsets
-        // support setting this to either 1 or 2, while others only support
-        // setting it to 1.
-        context->max_b_frames = 1;
-    }
-
     if( job->pass_id == HB_PASS_ENCODE_ANALYSIS ||
         job->pass_id == HB_PASS_ENCODE_FINAL )
     {
@@ -852,9 +854,7 @@ int encavcodecInit( hb_work_object_t * w, hb_job_t * job )
 
     if (context->extradata != NULL)
     {
-        memcpy(w->config->extradata.bytes, context->extradata,
-                                           context->extradata_size);
-        w->config->extradata.length = context->extradata_size;
+        hb_set_extradata(w->extradata, context->extradata, context->extradata_size);
     }
 
 done:
@@ -923,7 +923,7 @@ static void compute_dts_offset( hb_work_private_t * pv, hb_buffer_t * buf )
         if ( ( pv->frameno_in ) == pv->job->areBframes )
         {
             pv->dts_delay = buf->s.start;
-            pv->job->config.init_delay = pv->dts_delay;
+            pv->job->init_delay = pv->dts_delay;
         }
     }
 }
@@ -1204,6 +1204,21 @@ static int apply_encoder_options(hb_job_t *job, AVCodecContext *context, AVDicti
     return 0;
 }
 
+static int apply_vce_preset(AVDictionary **av_opts, const char *preset)
+{
+    if (preset)
+    {
+        if (!strcasecmp(preset, "balanced")
+            || !strcasecmp(preset, "speed")
+            || !strcasecmp(preset, "quality"))
+        {
+            av_opt_set(av_opts, "quality", preset, AV_OPT_SEARCH_CHILDREN);
+        }
+    }
+
+    return 0;
+}
+
 static int apply_vpx_preset(AVDictionary ** av_opts, const char * preset)
 {
     if (preset == NULL)
@@ -1276,8 +1291,22 @@ static int apply_vp9_10bit_preset(AVDictionary ** av_opts, const char * preset)
     return apply_vpx_preset(av_opts, preset);
 }
 
-static int apply_encoder_preset(int vcodec, AVDictionary ** av_opts,
-                                const char * preset)
+static int apply_ffv1_preset(AVCodecContext *context, AVDictionary **av_opts, const char *preset)
+{
+    if (!strcasecmp(preset, "preservation"))
+    {
+        context->gop_size = 1;
+        context->level = 3;
+        av_dict_set(av_opts, "coder", "1", 0);
+        av_dict_set(av_opts, "context", "1", 0);
+        av_dict_set(av_opts, "slicecrc", "1", 0);
+    }
+    return 0;
+}
+
+static int apply_encoder_preset(int vcodec, AVCodecContext *context,
+                                AVDictionary **av_opts,
+                                const char *preset)
 {
     switch (vcodec)
     {
@@ -1288,6 +1317,12 @@ static int apply_encoder_preset(int vcodec, AVDictionary ** av_opts,
         case HB_VCODEC_FFMPEG_VP9_10BIT:
             return apply_vp9_10bit_preset(av_opts, preset);
 
+        case HB_VCODEC_FFMPEG_VCE_H264:
+        case HB_VCODEC_FFMPEG_VCE_H265:
+        case HB_VCODEC_FFMPEG_VCE_H265_10BIT:
+        case HB_VCODEC_FFMPEG_VCE_AV1:
+            return apply_vce_preset(av_opts, preset);
+
 #if HB_PROJECT_FEATURE_NVENC
         case HB_VCODEC_FFMPEG_NVENC_H264:
         case HB_VCODEC_FFMPEG_NVENC_H265:
@@ -1298,6 +1333,9 @@ static int apply_encoder_preset(int vcodec, AVDictionary ** av_opts,
             av_dict_set( av_opts, "preset", preset, 0);
             break;
 #endif
+
+        case HB_VCODEC_FFMPEG_FFV1:
+            return apply_ffv1_preset(context, av_opts, preset);
         default:
             break;
     }
@@ -1305,6 +1343,97 @@ static int apply_encoder_preset(int vcodec, AVDictionary ** av_opts,
     return 0;
 }
 
+static int apply_vp9_tune(AVDictionary ** av_opts, const char * tune)
+{
+    av_dict_set(av_opts, "tune-content", tune, 0);
+    return 0;
+}
+
+static int apply_encoder_tune(int vcodec, AVDictionary ** av_opts,
+                                const char * tune)
+{
+    switch (vcodec)
+    {
+        case HB_VCODEC_FFMPEG_VP9:
+        case HB_VCODEC_FFMPEG_VP9_10BIT:
+            return apply_vp9_tune(av_opts, tune);
+        default:
+            break;
+    }
+
+    return 0;
+}
+
+static int apply_encoder_level(AVCodecContext *context, AVDictionary **av_opts, int vcodec, const char *encoder_level)
+{
+    const char * const *level_names = NULL;
+    const int  *level_values = NULL;
+
+    switch (vcodec)
+    {
+        case HB_VCODEC_FFMPEG_VCE_H264:
+        case HB_VCODEC_FFMPEG_NVENC_H264:
+        case HB_VCODEC_FFMPEG_MF_H264:
+            level_names = hb_h264_level_names;
+            level_values = hb_h264_level_values;
+            break;
+
+        case HB_VCODEC_FFMPEG_VCE_H265:
+        case HB_VCODEC_FFMPEG_VCE_H265_10BIT:
+        case HB_VCODEC_FFMPEG_NVENC_H265:
+        case HB_VCODEC_FFMPEG_NVENC_H265_10BIT:
+        case HB_VCODEC_FFMPEG_MF_H265:
+            level_names = hb_h265_level_names;
+            level_values = hb_h265_level_values;
+            break;
+
+        case HB_VCODEC_FFMPEG_VCE_AV1:
+        case HB_VCODEC_FFMPEG_NVENC_AV1:
+        case HB_VCODEC_FFMPEG_NVENC_AV1_10BIT:
+            level_names = hb_av1_level_names;
+            level_values = hb_av1_level_values;
+            break;
+
+        case HB_VCODEC_FFMPEG_FFV1:
+            level_names = hb_ffv1_level_names;
+            level_values = hb_ffv1_level_values;
+            break;
+    }
+
+    context->level = FF_LEVEL_UNKNOWN;
+
+    if (level_names == NULL || level_values == NULL)
+    {
+        return 0;
+    }
+
+    if (encoder_level != NULL && *encoder_level)
+    {
+        int i = 1;
+        while (level_names[i] != NULL)
+        {
+            if (!strcasecmp(encoder_level, level_names[i]))
+            {
+                if (vcodec == HB_VCODEC_FFMPEG_NVENC_H264 ||
+                    vcodec == HB_VCODEC_FFMPEG_NVENC_H265 ||
+                    vcodec == HB_VCODEC_FFMPEG_NVENC_H265_10BIT ||
+                    vcodec == HB_VCODEC_FFMPEG_NVENC_AV1 ||
+                    vcodec == HB_VCODEC_FFMPEG_NVENC_AV1_10BIT)
+                {
+                    av_dict_set(av_opts, "level", level_names[i], 0);
+                }
+                else
+                {
+                    context->level = level_values[i];
+                }
+            }
+            ++i;
+        }
+    }
+
+    return 0;
+}
+
 const char* const* hb_av_preset_get_names(int encoder)
 {
     switch (encoder)
@@ -1331,6 +1460,9 @@ const char* const* hb_av_preset_get_names(int encoder)
         case HB_VCODEC_FFMPEG_MF_H265:
             return h26x_mf_preset_name;
 
+        case HB_VCODEC_FFMPEG_FFV1:
+            return ffv1_preset_names;
+
         default:
             return NULL;
     }
@@ -1340,6 +1472,9 @@ const char* const* hb_av_tune_get_names(int encoder)
 {
     switch (encoder)
     {
+        case HB_VCODEC_FFMPEG_VP9:
+        case HB_VCODEC_FFMPEG_VP9_10BIT:
+            return vp9_tune_names;
         default:
             return NULL;
     }
@@ -1359,6 +1494,39 @@ const char* const* hb_av_profile_get_names(int encoder)
             return h264_mf_profile_name;
         case HB_VCODEC_FFMPEG_MF_H265:
             return h265_mf_profile_name;
+        case HB_VCODEC_FFMPEG_FFV1:
+            return ffv1_profile_names;
+
+         default:
+             return NULL;
+     }
+}
+
+const char* const* hb_av_level_get_names(int encoder)
+{
+    switch (encoder)
+    {
+        case HB_VCODEC_FFMPEG_NVENC_H264:
+        case HB_VCODEC_FFMPEG_MF_H264:
+            return hb_h264_level_names;
+
+     case HB_VCODEC_FFMPEG_VCE_H264:
+            return hb_vce_h264_level_names; // Not quite the same as x264
+
+        case HB_VCODEC_FFMPEG_NVENC_H265:
+        case HB_VCODEC_FFMPEG_NVENC_H265_10BIT:
+        case HB_VCODEC_FFMPEG_VCE_H265:
+        case HB_VCODEC_FFMPEG_VCE_H265_10BIT:
+        case HB_VCODEC_FFMPEG_MF_H265:
+            return hb_h265_level_names;
+
+        case HB_VCODEC_FFMPEG_VCE_AV1:
+        case HB_VCODEC_FFMPEG_NVENC_AV1:
+        case HB_VCODEC_FFMPEG_NVENC_AV1_10BIT:
+            return hb_av1_level_names;
+
+        case HB_VCODEC_FFMPEG_FFV1:
+            return hb_ffv1_level_names;
 
          default:
              return NULL;
@@ -1388,6 +1556,9 @@ const int* hb_av_get_pix_fmts(int encoder)
         case HB_VCODEC_FFMPEG_VP9_10BIT:
             return standard_10bit_pix_fmts;
 
+        case HB_VCODEC_FFMPEG_FFV1:
+            return ffv1_pix_formats;
+
          default:
              return standard_pix_fmts;
      }
