@@ -1,6 +1,6 @@
 /* json.c
 
-   Copyright (c) 2003-2023 HandBrake Team
+   Copyright (c) 2003-2024 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -910,7 +910,7 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
 
         audio_dict = json_pack_ex(&error, 0,
             "{s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o}",
-            "Track",                hb_value_int(audio->config.in.track),
+            "Track",                hb_value_int(audio->config.index),
             "Encoder",              hb_value_int(audio->config.out.codec),
             "Gain",                 hb_value_double(audio->config.out.gain),
             "DRC",                  hb_value_double(audio->config.out.dynamic_range_compression),
@@ -1535,7 +1535,7 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
             hb_audio_config_init(&audio);
             result = json_unpack_ex(audio_dict, &error, 0,
                 "{s:i, s?s, s?o, s?F, s?F, s?o, s?b, s?o, s?o, s?i, s?F, s?F}",
-                "Track",                unpack_i(&audio.in.track),
+                "Track",                unpack_i(&audio.index),
                 "Name",                 unpack_s(&name),
                 "Encoder",              unpack_o(&acodec),
                 "Gain",                 unpack_f(&audio.out.gain),
@@ -1607,7 +1607,7 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
             {
                 audio.out.name = name;
             }
-            if (audio.in.track >= 0)
+            if (audio.index >= 0)
             {
                 audio.out.track = ii;
                 hb_audio_add(job, &audio);
@@ -1897,118 +1897,6 @@ char* hb_set_anamorphic_size_json(const char * json_param)
     return result;
 }
 
-char* hb_get_preview_json(hb_handle_t * h, const char *json_param)
-{
-    hb_image_t *image;
-    int ii, title_idx, preview_idx, deinterlace = 0;
-
-    int json_result;
-    json_error_t error;
-    hb_dict_t * dict;
-    hb_geometry_settings_t settings;
-
-    // Clear dest geometry since some fields are optional.
-    memset(&settings, 0, sizeof(settings));
-
-    dict = hb_value_json(json_param);
-    json_result = json_unpack_ex(dict, &error, 0,
-    "{"
-    // Title, Preview, Deinterlace
-    "s:i, s:i, s?b,"
-    // DestSettings
-    "s:{"
-    //   Geometry {Width, Height, PAR {Num, Den}},
-    "s:{s:i, s:i, s:{s:i, s:i}},"
-    //   AnamorphicMode, Keep, ItuPAR, Modulus, MaxWidth, MaxHeight,
-    "s:i, s?i, s?b, s:i, s:i, s:i,"
-    //   Crop [Top, Bottom, Left, Right]
-    "s?[iiii]"
-    "  }"
-    "}",
-    "Title",                    unpack_i(&title_idx),
-    "Preview",                  unpack_i(&preview_idx),
-    "Deinterlace",              unpack_b(&deinterlace),
-    "DestSettings",
-        "Geometry",
-            "Width",            unpack_i(&settings.geometry.width),
-            "Height",           unpack_i(&settings.geometry.height),
-            "PAR",
-                "Num",          unpack_i(&settings.geometry.par.num),
-                "Den",          unpack_i(&settings.geometry.par.den),
-        "AnamorphicMode",       unpack_i(&settings.mode),
-        "Keep",                 unpack_i(&settings.keep),
-        "ItuPAR",               unpack_b(&settings.itu_par),
-        "Modulus",              unpack_i(&settings.modulus),
-        "MaxWidth",             unpack_i(&settings.maxWidth),
-        "MaxHeight",            unpack_i(&settings.maxHeight),
-        "Crop",                 unpack_i(&settings.crop[0]),
-                                unpack_i(&settings.crop[1]),
-                                unpack_i(&settings.crop[2]),
-                                unpack_i(&settings.crop[3])
-    );
-    hb_value_free(&dict);
-
-    if (json_result < 0)
-    {
-        hb_error("preview params: json unpack failure: %s", error.text);
-        return NULL;
-    }
-
-    image = hb_get_preview2(h, title_idx, preview_idx, &settings, deinterlace);
-    if (image == NULL)
-    {
-        return NULL;
-    }
-
-    dict = json_pack_ex(&error, 0,
-        "{s:o, s:o, s:o}",
-            "Format",       hb_value_int(image->format),
-            "Width",        hb_value_int(image->width),
-            "Height",       hb_value_int(image->height));
-    if (dict == NULL)
-    {
-        hb_error("hb_get_preview_json: pack failure: %s", error.text);
-        return NULL;
-    }
-
-    hb_value_array_t * planes = hb_value_array_init();
-    for (ii = 0; ii < 4; ii++)
-    {
-        int base64size = AV_BASE64_SIZE(image->plane[ii].size);
-        if (image->plane[ii].size <= 0 || base64size <= 0)
-            continue;
-
-        char *plane_base64 = calloc(base64size, 1);
-        av_base64_encode(plane_base64, base64size,
-                         image->plane[ii].data, image->plane[ii].size);
-
-        base64size = strlen(plane_base64);
-        hb_dict_t *plane_dict;
-        plane_dict = json_pack_ex(&error, 0,
-            "{s:o, s:o, s:o, s:o, s:o, s:o}",
-            "Width",        hb_value_int(image->plane[ii].width),
-            "Height",       hb_value_int(image->plane[ii].height),
-            "Stride",       hb_value_int(image->plane[ii].stride),
-            "HeightStride", hb_value_int(image->plane[ii].height_stride),
-            "Size",         hb_value_int(base64size),
-            "Data",         hb_value_string(plane_base64)
-        );
-        if (plane_dict == NULL)
-        {
-            hb_error("plane_dict: json pack failure: %s", error.text);
-            return NULL;
-        }
-        hb_value_array_append(planes, plane_dict);
-    }
-    hb_dict_set(dict, "Planes", planes);
-    hb_image_close(&image);
-
-    char *result = hb_value_get_json(dict);
-    hb_value_free(&dict);
-
-    return result;
-}
-
 hb_image_t * hb_get_preview3_json(hb_handle_t * h, int picture, const char *json_job)
 {
     hb_image_t * image;
