@@ -1,6 +1,6 @@
 /* scan.c
 
-   Copyright (c) 2003-2023 HandBrake Team
+   Copyright (c) 2003-2024 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -40,20 +40,17 @@ typedef struct
 } hb_scan_t;
 
 #define PREVIEW_READ_THRESH (200)
+#define AUDIO_DECODE_ERROR_LIMIT (10)
 
 static void ScanFunc( void * );
 static int  DecodePreviews( hb_scan_t *, hb_title_t * title, int flush );
-static void LookForAudio(hb_scan_t *scan, hb_title_t *title, hb_buffer_t *b);
+static hb_audio_t * find_audio_for_id(hb_title_t * title, int id);
+static void LookForAudio(hb_scan_t *scan, hb_title_t *title, hb_audio_t * audio, hb_buffer_t *b);
 static int  AllAudioOK( hb_title_t * title );
 static void UpdateState1(hb_scan_t *scan, int title);
 static void UpdateState2(hb_scan_t *scan, int title);
 static void UpdateState3(hb_scan_t *scan, int preview);
 
-static int get_color_prim(int color_primaries, hb_geometry_t geometry, hb_rational_t rate);
-static int get_color_transfer(int color_trc);
-static int get_color_matrix(int colorspace, hb_geometry_t geometry);
-static int get_color_range(int color_range);
-
 static const char *aspect_to_string(hb_rational_t *dar)
 {
     double aspect = (double)dar->num / dar->den;
@@ -70,140 +67,6 @@ static const char *aspect_to_string(hb_rational_t *dar)
     return arstr;
 }
 
-static int get_color_prim(int color_primaries, hb_geometry_t geometry, hb_rational_t rate)
-{
-    switch (color_primaries)
-    {
-        case AVCOL_PRI_BT709:
-            return HB_COLR_PRI_BT709;
-        case AVCOL_PRI_BT470M:
-            return HB_COLR_PRI_BT470M;
-        case AVCOL_PRI_BT470BG:
-            return HB_COLR_PRI_EBUTECH;
-        case AVCOL_PRI_SMPTE170M:
-        case AVCOL_PRI_SMPTE240M:
-            return HB_COLR_PRI_SMPTEC;
-        case AVCOL_PRI_FILM:
-            return HB_COLR_PRI_FILM;
-        case AVCOL_PRI_SMPTE428:
-            return HB_COLR_PRI_SMPTE428;
-        case AVCOL_PRI_SMPTE431:
-            return HB_COLR_PRI_SMPTE431;
-        case AVCOL_PRI_SMPTE432:
-            return HB_COLR_PRI_SMPTE432;
-        case AVCOL_PRI_JEDEC_P22:
-            return HB_COLR_PRI_JEDEC_P22;
-        case AVCOL_PRI_BT2020:
-            return HB_COLR_PRI_BT2020;
-        default:
-        {
-            if ((geometry.width >= 1280 || geometry.height >= 720)||
-                (geometry.width >   720 && geometry.height >  576 ))
-                // ITU BT.709 HD content
-                return HB_COLR_PRI_BT709;
-            else if (rate.den == 1080000)
-                // ITU BT.601 DVD or SD TV content (PAL)
-                return HB_COLR_PRI_EBUTECH;
-            else
-                // ITU BT.601 DVD or SD TV content (NTSC)
-                return HB_COLR_PRI_SMPTEC;
-        }
-    }
-}
-
-static int get_color_transfer(int color_trc)
-{
-    switch (color_trc)
-    {
-        case AVCOL_TRC_GAMMA22:
-            return HB_COLR_TRA_GAMMA22;
-        case AVCOL_TRC_GAMMA28:
-            return HB_COLR_TRA_GAMMA28;
-        case AVCOL_TRC_SMPTE170M:
-            return HB_COLR_TRA_SMPTE170M;
-        case AVCOL_TRC_LINEAR:
-            return HB_COLR_TRA_LINEAR;
-        case AVCOL_TRC_LOG:
-            return HB_COLR_TRA_LOG;
-        case AVCOL_TRC_LOG_SQRT:
-            return HB_COLR_TRA_LOG_SQRT;
-        case AVCOL_TRC_IEC61966_2_4:
-            return HB_COLR_TRA_IEC61966_2_4;
-        case AVCOL_TRC_BT1361_ECG:
-            return HB_COLR_TRA_BT1361_ECG;
-        case AVCOL_TRC_IEC61966_2_1:
-            return HB_COLR_TRA_IEC61966_2_1;
-        case AVCOL_TRC_SMPTE240M:
-            return HB_COLR_TRA_SMPTE240M;
-        case AVCOL_TRC_SMPTEST2084:
-            return HB_COLR_TRA_SMPTEST2084;
-        case AVCOL_TRC_ARIB_STD_B67:
-            return HB_COLR_TRA_ARIB_STD_B67;
-        case AVCOL_TRC_BT2020_10:
-            return HB_COLR_TRA_BT2020_10;
-        case AVCOL_TRC_BT2020_12:
-            return HB_COLR_TRA_BT2020_12;
-        default:
-            // ITU BT.601, BT.709, anything else
-            return HB_COLR_TRA_BT709;
-    }
-}
-
-static int get_color_matrix(int colorspace, hb_geometry_t geometry)
-{
-    switch (colorspace)
-    {
-        case AVCOL_SPC_RGB:
-            return HB_COLR_MAT_RGB;
-        case AVCOL_SPC_BT709:
-            return HB_COLR_MAT_BT709;
-        case AVCOL_SPC_FCC:
-            return HB_COLR_MAT_FCC;
-        case AVCOL_SPC_BT470BG:
-            return HB_COLR_MAT_BT470BG;
-        case AVCOL_SPC_SMPTE170M:
-            return HB_COLR_MAT_SMPTE170M;
-        case AVCOL_SPC_SMPTE240M:
-            return HB_COLR_MAT_SMPTE240M;
-        case AVCOL_SPC_YCGCO:
-            return HB_COLR_MAT_YCGCO;
-        case AVCOL_SPC_BT2020_NCL:
-            return HB_COLR_MAT_BT2020_NCL;
-        case AVCOL_SPC_BT2020_CL:
-            return HB_COLR_MAT_BT2020_CL;
-        case AVCOL_SPC_CHROMA_DERIVED_NCL:
-            return HB_COLR_MAT_CD_NCL;
-        case AVCOL_SPC_CHROMA_DERIVED_CL:
-            return HB_COLR_MAT_CD_CL;
-        case AVCOL_SPC_ICTCP:
-            return HB_COLR_MAT_ICTCP;
-        default:
-        {
-            if ((geometry.width >= 1280 || geometry.height >= 720)||
-                (geometry.width >   720 && geometry.height >  576 ))
-                // ITU BT.709 HD content
-                return HB_COLR_MAT_BT709;
-            else
-                // ITU BT.601 DVD or SD TV content (PAL)
-                // ITU BT.601 DVD or SD TV content (NTSC)
-                return HB_COLR_MAT_SMPTE170M;
-        }
-    }
-}
-
-static int get_color_range(int color_range)
-{
-    switch (color_range)
-    {
-        case AVCOL_RANGE_MPEG:
-            return AVCOL_RANGE_MPEG;
-        case AVCOL_RANGE_JPEG:
-            return AVCOL_RANGE_JPEG;
-        default:
-            return AVCOL_RANGE_MPEG;
-    }
-}
-
 static const char * const known_file_types[] =
 {
     "mp4", "m4v", "mov", "flv", "mkv", "avi", "webm", "wmv",  NULL
@@ -345,6 +208,11 @@ static void ScanFunc( void * _data )
             /* Scan all titles */
             for( i = 0; i < hb_batch_title_count( data->batch ); i++ )
             {
+                if (*data->die)
+                {
+                    goto finish;
+                }
+
                 hb_title_t * title;
 
                 UpdateState1(data, i + 1);
@@ -361,6 +229,11 @@ static void ScanFunc( void * _data )
         // If dragging a batch of files, maybe not, but if the UI's implement a recursive folder maybe?
         for (i = 0; i < hb_list_count( data->paths ); i++)
         {
+            if (*data->die)
+            {
+                goto finish;
+            }
+
             single_path = hb_list_item(data->paths, i);
 
             UpdateState1(data, i + 1);
@@ -802,14 +675,18 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
         title->angle_count = hb_dvd_angle_count( data->dvd );
         hb_log( "scan: title angle(s) %d", title->angle_count );
     }
-    else if (data->batch)
+    else // data->batch or a single file
     {
         stream = hb_stream_open(data->h, title->path, title, 0);
     }
-    else 
+
+    if (data->bd == NULL && data->dvd == NULL && stream == NULL)
     {
-        // We have a batch of files.
-        stream = hb_stream_open(data->h, title->path, title, 0);
+        hb_error("Can't open stream!");
+        free(info_list);
+        crop_record_free(crops);
+        hb_stream_close(&stream);
+        return 0;
     }
 
     if (title->video_codec == WORK_NONE)
@@ -852,6 +729,7 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
         crop_record_free(crops);
         free( vid_decoder );
         hb_stream_close(&stream);
+        hb_hwaccel_hw_ctx_close(&hw_device_ctx);
         return 0;
     }
 
@@ -868,6 +746,7 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
             vid_decoder->close( vid_decoder );
             free( vid_decoder );
             hb_stream_close(&stream);
+            hb_hwaccel_hw_ctx_close(&hw_device_ctx);
             return 0;
         }
         if (data->bd)
@@ -1031,8 +910,12 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
                 }
                 else if (!AllAudioOK(title) && !abort_audio)
                 {
-                    LookForAudio( data, title, buf_es );
-                    buf_es = NULL;
+                    hb_audio_t * audio = find_audio_for_id(title, buf_es->s.id);
+                    if (audio != NULL && audio->priv.scan_error_count < AUDIO_DECODE_ERROR_LIMIT)
+                    {
+                        LookForAudio( data, title, audio, buf_es );
+                        buf_es = NULL;
+                    }
                 }
                 if ( buf_es )
                     hb_buffer_close( &buf_es );
@@ -1302,27 +1185,40 @@ skip_preview:
         }
         title->pix_fmt = vid_info.pix_fmt;
 
-        if ((title->color_prim     != HB_COLR_PRI_UNDEF &&
-             title->color_prim     != HB_COLR_PRI_UNSET) ||
-            (title->color_transfer != HB_COLR_TRA_UNDEF &&
-             title->color_transfer != HB_COLR_TRA_UNSET) ||
-            (title->color_matrix   != HB_COLR_MAT_UNDEF &&
-             title->color_matrix != HB_COLR_MAT_UNSET))
+        // DVD-Video have no defined color info, but some mostly wrong
+        // values could be read from the mpeg-2 bitstream. Override those here.
+        if (data->dvd)
         {
-            title->color_prim     = get_color_prim(title->color_prim, vid_info.geometry, vid_info.rate);
-            title->color_transfer = get_color_transfer(title->color_transfer);
-            title->color_matrix   = get_color_matrix(title->color_matrix, vid_info.geometry);
+            title->color_prim     = hb_get_color_prim(HB_COLR_PRI_UNSET, vid_info.geometry, vid_info.rate);
+            title->color_transfer = HB_COLR_TRA_BT709;
+            title->color_matrix   = HB_COLR_MAT_SMPTE170M;
         }
-        // Let's try to guess a color profile only if the source is not Dolby Vision 5
-        // which requires the values to be unset
-        else if (title->dovi.dv_profile != 5)
+        else if ((title->color_prim     != HB_COLR_PRI_UNDEF &&
+                  title->color_prim     != HB_COLR_PRI_UNSET) ||
+                 (title->color_transfer != HB_COLR_TRA_UNDEF &&
+                  title->color_transfer != HB_COLR_TRA_UNSET) ||
+                 (title->color_matrix   != HB_COLR_MAT_UNDEF &&
+                  title->color_matrix != HB_COLR_MAT_UNSET))
         {
-            title->color_prim     = get_color_prim(vid_info.color_prim, vid_info.geometry, vid_info.rate);
-            title->color_transfer = get_color_transfer(vid_info.color_transfer);
-            title->color_matrix   = get_color_matrix(vid_info.color_matrix, vid_info.geometry);
+            title->color_prim     = hb_get_color_prim(title->color_prim, vid_info.geometry, vid_info.rate);
+            title->color_transfer = hb_get_color_transfer(title->color_transfer);
+            title->color_matrix   = hb_get_color_matrix(title->color_matrix, vid_info.geometry);
+        }
+        else if (title->dovi.dv_profile == 5 ||
+                 (title->dovi.dv_profile == 10 && title->dovi.dv_bl_signal_compatibility_id == 0))
+        {
+            title->color_prim     = HB_COLR_PRI_UNDEF;
+            title->color_transfer = HB_COLR_TRA_UNDEF;
+            title->color_matrix   = HB_COLR_MAT_UNDEF;
+        }
+        else
+        {
+            title->color_prim     = hb_get_color_prim(vid_info.color_prim, vid_info.geometry, vid_info.rate);
+            title->color_transfer = hb_get_color_transfer(vid_info.color_transfer);
+            title->color_matrix   = hb_get_color_matrix(vid_info.color_matrix, vid_info.geometry);
         }
 
-        title->color_range = get_color_range(vid_info.color_range);
+        title->color_range = hb_get_color_range(vid_info.color_range);
         title->chroma_location = vid_info.chroma_location;
 
         title->video_decode_support = vid_info.video_decode_support;
@@ -1466,7 +1362,7 @@ skip_preview:
 
         if (title->ambient.ambient_illuminance.num)
         {
-            hb_log("scan: ambient viewing environment: ambient_illuminance=%f, ambient_light_x=%f, ambient_light_x=%f",
+            hb_log("scan: ambient viewing environment: ambient_illuminance=%f, ambient_light_x=%f, ambient_light_y=%f",
                    hb_q2d(title->ambient.ambient_illuminance),
                    hb_q2d(title->ambient.ambient_light_x),
                    hb_q2d(title->ambient.ambient_light_y));
@@ -1522,6 +1418,23 @@ skip_preview:
     return npreviews;
 }
 
+static hb_audio_t * find_audio_for_id(hb_title_t * title, int id)
+{
+    int i;
+    hb_audio_t * audio = NULL;
+
+    for( i = 0; i < hb_list_count( title->list_audio ); i++ )
+    {
+        audio = hb_list_item( title->list_audio, i );
+        /* check if this elementary stream is one we want */
+        if ( audio->id == id )
+        {
+            return audio;
+        }
+    }
+    return NULL;
+}
+
 /*
  * This routine is called for every frame from a non-video elementary stream.
  * These are a mix of audio & subtitle streams, some of which we want & some
@@ -1536,24 +1449,8 @@ skip_preview:
  * aren't (e.g., some European DVD Teletext streams use the same IDs as US ATSC
  * AC-3 audio).
  */
-static void LookForAudio(hb_scan_t *scan, hb_title_t * title, hb_buffer_t * b)
+static void LookForAudio(hb_scan_t *scan, hb_title_t * title, hb_audio_t * audio, hb_buffer_t * b)
 {
-    int i;
-
-    hb_audio_t * audio = NULL;
-    for( i = 0; i < hb_list_count( title->list_audio ); i++ )
-    {
-        audio = hb_list_item( title->list_audio, i );
-        /* check if this elementary stream is one we want */
-        if ( audio->id == b->s.id )
-        {
-            break;
-        }
-        else
-        {
-            audio = NULL;
-        }
-    }
     if( !audio || audio->config.in.bitrate != 0 )
     {
         /* not found or already done */
@@ -1587,16 +1484,22 @@ static void LookForAudio(hb_scan_t *scan, hb_title_t * title, hb_buffer_t * b)
     w->codec_param = audio->config.in.codec_param;
     b = hb_fifo_see( audio->priv.scan_cache );
     int ret = w->bsinfo( w, b, &info );
-    if ( ret < 0 )
+    if ( ret < 0 && ret != AVERROR(EAGAIN) )
     {
-        hb_log( "no info on audio type %d/0x%x for id 0x%x",
-                audio->config.in.codec, audio->config.in.codec_param,
-                audio->id );
-        goto drop_audio;
+        // No point in attempting to decode the
+        // same data again the next time around
+        hb_buffer_t * tmp;
+        tmp = hb_fifo_get( audio->priv.scan_cache );
+        hb_buffer_close( &tmp );
+        free( w );
+        audio->priv.scan_error_count++;
+        return;
     }
     if ( !info.bitrate )
     {
-        /* didn't find any info */
+        // didn't find any info
+        // Additional buffer data may be required to obtain
+        // audio attributes
         free( w );
         return;
     }
@@ -1643,7 +1546,7 @@ static void LookForAudio(hb_scan_t *scan, hb_title_t * title, hb_buffer_t * b)
                     codec_name = "TrueHD";
                     break;
                 case AV_CODEC_ID_DTS:
-                    if (profile_name == NULL &&
+                    if ((profile_name == NULL || info.profile == AV_PROFILE_DTS) &&
                         audio->config.in.codec == HB_ACODEC_DCA_HD)
                         codec_name = "DTS-HD";
                     else
@@ -1860,7 +1763,7 @@ static int  AllAudioOK( hb_title_t * title )
     for( i = 0; i < hb_list_count( title->list_audio ); i++ )
     {
         audio = hb_list_item( title->list_audio, i );
-        if( audio->config.in.bitrate == 0 )
+        if ( audio->priv.scan_error_count < AUDIO_DECODE_ERROR_LIMIT && audio->config.in.bitrate == 0 )
         {
             return 0;
         }
